  {% if post.published and post.date != post.update %}
  <li>{% if fa %}<i class="fa fa-pencil-square-o"></i>{% else %}{{ gettext('Update') }}:{% endif %}
    {{ post.update.strftime(gettext(ablog.post_date_format)) }}</li>
  {% endif %}

  {% if post.author %}
  <li>{% if fa %}<i class="fa-fw fa fa-user"></i>{% else %}{{ gettext('Author') }}:{% endif %}
    {% for coll in post.author %}
      {% if coll|length %}
      <a href="{{ pathto(coll.docname) }}">{{ coll }}</a>{% if loop.index < post.author|length %},{% endif %}
      {% else %}{{ coll }}{% if loop.index < post.author|length %},{% endif %}{% endif %}
    {% endfor %}</li>
  {% endif %}

  {% if post.location %}
  <li>{% if fa %}<i class="fa-fw fa fa-location-arrow"></i>{% else %}{{ gettext('Location') }}:{% endif %}
    {% for coll in post.location %}
      {% if coll|length %}
      <a href="{{ pathto(coll.docname) }}">{{ coll }}</a>{% if loop.index < post.location|length %},{% endif %}
      {% else %}{{ coll }}{% if loop.index < post.location|length %},{% endif %}{% endif %}
    {% endfor %}</li>
  {% endif %}

  {% if post.language %}
  <li>{% if fa %}<i class="fa-fw fa fa-language"></i>{% else %}{{ gettext('Language') }}:{% endif %}
    {% for coll in post.language %}
      {% if coll|length %}
      <a href="{{ pathto(coll.docname) }}">{{ coll }}</a>{% if loop.index < post.language|length %},{% endif %}
      {% else %}{{ coll }}{% if loop.index < post.language|length %},{% endif %}{% endif %}
    {% endfor %}</li>
  {% endif %}

  {% if post.category %}
  <li>{% if fa %}<i class="fa-fw fa fa-folder-open"></i>{% else %}{{ gettext('Category') }}:{% endif %}
    {% for coll in post.category %}
      {% if coll|length %}
      <a href="{{ pathto(coll.docname) }}">{{ coll }}</a>{% if loop.index < post.category|length %},{% endif %}
      {% else %}{{ coll }}{% if loop.index < post.category|length %},{% endif %}{% endif %}
    {% endfor %}</li>
  {% endif %}

  {% if post.tags %}
  <li>{% if post.tags|length > 1 %}{% if fa %}<i class="fa-fw fa fa-tags"></i>{% else %}{{ gettext('Tags') }}:{% endif %}
      {% else %}{% if fa %}<i class="fa-fw fa fa-tag"></i>{% else %}{{ gettext('Tag') }}:{% endif %}{% endif %}
    {% for coll in post.tags %}
      {% if coll|length %}
      <a href="{{ pathto(coll.docname) }}">{{ coll }}</a>{% if loop.index < post.tags|length %},{% endif %}
      {% else %}{{ coll }}{% if loop.index < post.tags|length %},{% endif %}{% endif %}
    {% endfor %}</li>
  {% endif %}
  {% if ablog.disqus_shortname and (ablog[pagename].published or ablog.disqus_drafts) %}
  <li>
    <script type="text/javascript">
    /* * * CONFIGURATION VARIABLES: EDIT BEFORE PASTING INTO YOUR WEBPAGE * * */
    var disqus_shortname = '{{ ablog.disqus_shortname }}'; // required: replace example with your forum shortname

    /* * * DON'T EDIT BELOW THIS LINE * * */
    (function () {
    var s = document.createElement('script'); s.async = true;
    s.type = 'text/javascript';
    s.src = 'http://' + disqus_shortname + '.disqus.com/count.js';
    (document.getElementsByTagName('HEAD')[0] || document.getElementsByTagName('BODY')[0]).appendChild(s);
    }());
    </script>
    {% if fa %}<i class="fa-fw fa fa-comments"></i>{% endif %}
    <a href="{% if pagename != post.docname%}{{ pathto(post.docname) }}{% endif %}#disqus_thread" data-disqus-identifier="/{{post.docname}}/"> {% if not fa %}comment(s){% endif %}</a>
  </li>
  {% endif %}